# -*- coding: utf-8 -*-
#
# Copyright 2004-2006 André Malo or his licensors, as applicable
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""
Test runner for the svnmailer test suite
"""
__author__    = "André Malo"
__docformat__ = "epytext en"
__all__       = ['getSuites']


def getSuites(argv = ()):
    """ Returns the test suites in svnmailer_test.suite

        @param argv: A list of suites to run
        @type argv: C{tuple} or C{list}

        @return: The test suite list; each list item is a tuple, where
            the first item is the test module object and the second item
            the suite object.
        @rtype: C{list}
    """
    from svnmailer_test import suite

    if argv:
        return [
            (suite._suitedict_[name], suite._suitedict_[name].getSuite())
            for name in suite._suites_ if name in argv
        ]
    else:
        return [(module, module.getSuite()) for module in suite._modules_]


def run():
    """ Run the tests """
    import os, sys, unittest
    
    try:
        isatty = os.isatty(sys.stdout.fileno())
    except AttributeError:
        isatty = False

    if isatty and sys.platform != 'win32':
        h_begin, h_end = ("\033[1m", "\033[0m")
    else:
        h_begin, h_end = ("", "")

    errors = 0
    success = 0
    rerun = []

    for suite in getSuites(sys.argv[1:]):
        print "%(begin)s>>> Running testsuite %(suite)s%(end)s" % {
            'suite': suite[0].__name__,
            'begin': h_begin,
            'end':   h_end,
        }

        result = unittest.TextTestRunner(
            stream = sys.stdout,
            verbosity = 2
        ).run(suite[1])

        if result.wasSuccessful():
            success += result.testsRun
        else:
            this_errors = len(result.errors) + len(result.failures)
            errors += this_errors
            success += result.testsRun - this_errors
            rerun.append(suite[0].__name__)

        print

    print "====================================="
    print "Success: %d" % success
    print "Failure: %d" % errors
    if rerun:
        print "Failed suites:\n  %s" % '\n  '.join([
            suite.replace('svnmailer_test.suite.', '', 1) for suite in rerun
        ])


if __name__ == '__main__':
    run()
